# Copyright (C) 2020 Ludvig Larsson aka Valmond ludviglarsson@protonmail.com
# This code is licensed under the terms of the GNU Public License, version 3 or later (GPL-3.0-or-later)

from Cryptodome.Cipher import AES
from Cryptodome import Random
from Cryptodome.Util import Counter
import binascii


# Symmetric encryption/decryption for arbitrary length data
# Makes a random key if not specified
class CipherHelper:
    def __init__(self, aes_key=None, raw_iv=None):
        if aes_key:
            self.aes_key = aes_key
            self.raw_iv = raw_iv
        else:
            # Make a new fresh AES keypair for this communication:
            key_bytes = 32
            self.aes_key = Random.new().read(key_bytes)
            # Choose a random, 16-byte IV.
            self.raw_iv = Random.new().read(AES.block_size)

        # Make a python integer from raw_iv
        self._aes_iv = int(binascii.hexlify(self.raw_iv), 16)

    def encrypt(self, plaintext):
        # Create a new Counter object with IV = iv_int.
        ctr = Counter.new(AES.block_size * 8, initial_value=self._aes_iv)
        # Create AES-CTR cipher.
        aes = AES.new(self.aes_key, AES.MODE_CTR, counter=ctr)
        # Encrypt and return IV and ciphertext.
        return aes.encrypt(plaintext)

    def decrypt(self, encrypted_message):
        # Initialize counter for decryption. iv should be the same as the output of encrypt().
        ctr = Counter.new(AES.block_size * 8, initial_value=self._aes_iv)
        # Create AES-CTR cipher.
        aes = AES.new(self.aes_key, AES.MODE_CTR, counter=ctr)
        # Decrypt
        return aes.decrypt(encrypted_message)
